@@ -9,7 +9,7 @@
 from __future__ import annotations
 
 
-DOCUMENTATION = '''
+DOCUMENTATION = """
 ---
 module: apt_repository
 short_description: Add and remove APT repositories
@@ -88,8 +88,8 @@ options:
         description:
             - Whether to automatically try to install the Python apt library or not, if it is not already installed.
               Without this library, the module does not work.
-            - Runs C(apt-get install python-apt) for Python 2, and C(apt-get install python3-apt) for Python 3.
-            - Only works with the system Python 2 or Python 3. If you are using a Python on the remote that is not
+            - Runs C(apt-get install python3-apt).
+            - Only works with the system Python. If you are using a Python on the remote that is not
               the system Python, set O(install_python_apt=false) and ensure that the Python apt library
               for your Python version is installed some other way.
         type: bool
@@ -98,12 +98,11 @@ author:
 - Alexander Saltanov (@sashka)
 version_added: "0.7"
 requirements:
-   - python-apt (python 2)
-   - python3-apt (python 3)
+   - python3-apt
    - apt-key or gpg
-'''
+"""
 
-EXAMPLES = '''
+EXAMPLES = """
 - name: Add specified repository into sources list
   ansible.builtin.apt_repository:
     repo: deb http://archive.canonical.com/ubuntu hardy partner
@@ -143,11 +142,11 @@ EXAMPLES = '''
 
     - name: somerepo | apt source
       ansible.builtin.apt_repository:
-        repo: "deb [arch=amd64 signed-by=/etc/apt/keyrings/myrepo.asc] https://download.example.com/linux/ubuntu {{ ansible_distribution_release }} stable"
+        repo: "deb [arch=amd64 signed-by=/etc/apt/keyrings/somerepo.asc] https://download.example.com/linux/ubuntu {{ ansible_distribution_release }} stable"
         state: present
-'''
+"""
 
-RETURN = '''
+RETURN = """
 repo:
   description: A source string for the repository
   returned: always
@@ -167,7 +166,7 @@ sources_removed:
   type: list
   sample: ["/etc/apt/sources.list.d/artifacts_elastic_co_packages_6_x_apt.list"]
   version_added: "2.15"
-'''
+"""
 
 import copy
 import glob
@@ -232,20 +231,21 @@ class SourcesList(object):
         self.files_mapping = {}  # internal DS for tracking symlinks
         # Repositories that we're adding -- used to implement mode param
         self.new_repos = set()
-        self.default_file = self._apt_cfg_file('Dir::Etc::sourcelist')
+        self.default_file = apt_pkg.config.find_file('Dir::Etc::sourcelist')
 
         # read sources.list if it exists
         if os.path.isfile(self.default_file):
             self.load(self.default_file)
 
         # read sources.list.d
-        for file in glob.iglob('%s/*.list' % self._apt_cfg_dir('Dir::Etc::sourceparts')):
+        self.sources_dir = apt_pkg.config.find_dir('Dir::Etc::sourceparts')
+        for file in glob.iglob(f'{self.sources_dir}/*.list'):
             if os.path.islink(file):
                 self.files_mapping[file] = os.readlink(file)
             self.load(file)
 
     def __iter__(self):
-        '''Simple iterator to go over all sources. Empty, non-source, and other not valid lines will be skipped.'''
+        """Simple iterator to go over all sources. Empty, non-source, and other not valid lines will be skipped."""
         for file, sources in self.files.items():
             for n, valid, enabled, source, comment in sources:
                 if valid:
@@ -255,7 +255,7 @@ class SourcesList(object):
         if '/' in filename:
             return filename
         else:
-            return os.path.abspath(os.path.join(self._apt_cfg_dir('Dir::Etc::sourceparts'), filename))
+            return os.path.abspath(os.path.join(self.sources_dir, filename))
 
     def _suggest_filename(self, line):
         def _cleanup_filename(s):
@@ -313,28 +313,6 @@ class SourcesList(object):
 
         return valid, enabled, source, comment
 
-    @staticmethod
-    def _apt_cfg_file(filespec):
-        '''
-        Wrapper for `apt_pkg` module for running with Python 2.5
-        '''
-        try:
-            result = apt_pkg.config.find_file(filespec)
-        except AttributeError:
-            result = apt_pkg.Config.FindFile(filespec)
-        return result
-
-    @staticmethod
-    def _apt_cfg_dir(dirspec):
-        '''
-        Wrapper for `apt_pkg` module for running with Python 2.5
-        '''
-        try:
-            result = apt_pkg.config.find_dir(dirspec)
-        except AttributeError:
-            result = apt_pkg.Config.FindDir(dirspec)
-        return result
-
     def load(self, file):
         group = []
         f = open(file, 'r')
@@ -355,8 +333,8 @@ class SourcesList(object):
 
                 try:
                     fd, tmp_path = tempfile.mkstemp(prefix=".%s-" % fn, dir=d)
-                except (OSError, IOError) as e:
-                    self.module.fail_json(msg='Unable to create temp file at "%s" for apt source: %s' % (d, to_native(e)))
+                except OSError as ex:
+                    raise Exception(f'Unable to create temp file at {d!r} for apt source.') from ex
 
                 f = os.fdopen(fd, 'w')
                 for n, valid, enabled, source, comment in sources:
@@ -372,8 +350,8 @@ class SourcesList(object):
 
                     try:
                         f.write(line)
-                    except IOError as ex:
-                        self.module.fail_json(msg="Failed to write to file %s: %s" % (tmp_path, to_native(ex)))
+                    except OSError as ex:
+                        raise Exception(f"Failed to write to file {tmp_path!r}.") from ex
                 if filename in self.files_mapping:
                     # Write to symlink target instead of replacing symlink as a normal file
                     self.module.atomic_move(tmp_path, self.files_mapping[filename])
@@ -413,10 +391,10 @@ class SourcesList(object):
         return new
 
     def modify(self, file, n, enabled=None, source=None, comment=None):
-        '''
+        """
         This function to be used with iterator, so we don't care of invalid sources.
         If source, enabled, or comment is None, original value from line ``n`` will be preserved.
-        '''
+        """
         valid, enabled_old, source_old, comment_old = self.files[file][n][1:]
         self.files[file][n] = (n, valid, self._choice(enabled, enabled_old), self._choice(source, source_old), self._choice(comment, comment_old))
 
@@ -475,7 +453,10 @@ class UbuntuSourcesList(SourcesList):
         self.apt_key_bin = self.module.get_bin_path('apt-key', required=False)
         self.gpg_bin = self.module.get_bin_path('gpg', required=False)
         if not self.apt_key_bin and not self.gpg_bin:
-            self.module.fail_json(msg='Either apt-key or gpg binary is required, but neither could be found')
+            msg = 'Either apt-key or gpg binary is required, but neither could be found.' \
+                  'The apt-key CLI has been deprecated and removed in modern Debian and derivatives, ' \
+                  'you might want to use "deb822_repository" instead.'
+            self.module.fail_json(msg)
 
     def __deepcopy__(self, memo=None):
         return UbuntuSourcesList(self.module)
@@ -526,8 +507,8 @@ class UbuntuSourcesList(SourcesList):
             if os.path.exists(key_file):
                 try:
                     rc, out, err = self.module.run_command([self.gpg_bin, '--list-packets', key_file])
-                except (IOError, OSError) as e:
-                    self.debug("Could check key against file %s: %s" % (key_file, to_native(e)))
+                except OSError as ex:
+                    self.module.debug(f"Could check key against file {key_file!r}: {ex}")
                     continue
 
                 if key_fingerprint in out:
@@ -576,8 +557,8 @@ class UbuntuSourcesList(SourcesList):
                             with open(keyfile, 'wb') as f:
                                 f.write(stdout)
                             self.module.log('Added repo key "%s" for apt to file "%s"' % (info['signing_key_fingerprint'], keyfile))
-                        except (OSError, IOError) as e:
-                            self.module.fail_json(msg='Unable to add required signing key for%s ', rc=rc, stderr=stderr, error=to_native(e))
+                        except OSError as ex:
+                            self.module.fail_json(msg='Unable to add required signing key.', rc=rc, stderr=stderr, error=str(ex), exception=ex)
 
             # apt source file
             file = file or self._suggest_filename('%s_%s' % (line, self.codename))
@@ -616,7 +597,7 @@ class UbuntuSourcesList(SourcesList):
 
 
 def revert_sources_list(sources_before, sources_after, sourceslist_before):
-    '''Revert the sourcelist files to their previous state.'''
+    """Revert the sourcelist files to their previous state."""
 
     # First remove any new files that were created:
     for filename in set(sources_after.keys()).difference(sources_before.keys()):
@@ -771,9 +752,9 @@ def main():
                     )
                     module.fail_json(msg=msg)
 
-        except (OSError, IOError) as ex:
+        except OSError as ex:
             revert_sources_list(sources_before, sources_after, sourceslist_before)
-            module.fail_json(msg=to_native(ex))
+            raise
 
     module.exit_json(changed=changed, repo=repo, sources_added=sources_added, sources_removed=sources_removed, state=state, diff=diff)
 
