@@ -6,7 +6,7 @@
 from __future__ import annotations
 
 
-DOCUMENTATION = '''
+DOCUMENTATION = """
 ---
 module: pip
 short_description: Manages Python library dependencies
@@ -28,21 +28,21 @@ options:
   requirements:
     description:
       - The path to a pip requirements file, which should be local to the remote system.
-        File can be specified as a relative path if using the chdir option.
+        File can be specified as a relative path if using the O(chdir) option.
     type: str
   virtualenv:
     description:
       - An optional path to a I(virtualenv) directory to install into.
-        It cannot be specified together with the 'executable' parameter
+        It cannot be specified together with the O(executable) parameter
         (added in 2.1).
         If the virtualenv does not exist, it will be created before installing
-        packages. The optional virtualenv_site_packages, virtualenv_command,
-        and virtualenv_python options affect the creation of the virtualenv.
+        packages. The optional O(virtualenv_site_packages), O(virtualenv_command),
+        and O(virtualenv_python) options affect the creation of the virtualenv.
     type: path
   virtualenv_site_packages:
     description:
       - Whether the virtual environment will inherit packages from the
-        global site-packages directory.  Note that if this setting is
+        global C(site-packages) directory. Note that if this setting is
         changed on an already existing virtual environment it will not
         have any effect, the environment must be deleted and newly
         created.
@@ -60,7 +60,7 @@ options:
   virtualenv_python:
     description:
       - The Python executable used for creating the virtual environment.
-        For example V(python3.12), V(python2.7). When not specified, the
+        For example V(python3.13). When not specified, the
         Python version used to run the ansible module is used. This parameter
         should not be used when O(virtualenv_command) is using V(pyvenv) or
         the C(-m venv) module.
@@ -68,14 +68,14 @@ options:
     version_added: "2.0"
   state:
     description:
-      - The state of module
-      - The 'forcereinstall' option is only available in Ansible 2.1 and above.
+      - The state of module.
+      - The V(forcereinstall) option is only available in Ansible 2.1 and above.
     type: str
     choices: [ absent, forcereinstall, latest, present ]
     default: present
   extra_args:
     description:
-      - Extra arguments passed to pip.
+      - Extra arguments passed to C(pip).
     type: str
     version_added: "1.0"
   editable:
@@ -86,18 +86,18 @@ options:
     version_added: "2.0"
   chdir:
     description:
-      - cd into this directory before running the command
+      - cd into this directory before running the command.
     type: path
     version_added: "1.3"
   executable:
     description:
-      - The explicit executable or pathname for the pip executable,
+      - The explicit executable or pathname for the C(pip) executable,
         if different from the Ansible Python interpreter. For
-        example V(pip3.3), if there are both Python 2.7 and 3.3 installations
-        in the system and you want to run pip for the Python 3.3 installation.
+        example V(pip3.13), if there are multiple Python installations
+        in the system and you want to run pip for the Python 3.13 installation.
       - Mutually exclusive with O(virtualenv) (added in 2.1).
       - Does not affect the Ansible Python interpreter.
-      - The setuptools package must be installed for both the Ansible Python interpreter
+      - The C(setuptools) package must be installed for both the Ansible Python interpreter
         and for the version of Python specified by this option.
     type: path
     version_added: "1.3"
@@ -105,14 +105,14 @@ options:
     description:
       - The system umask to apply before installing the pip package. This is
         useful, for example, when installing on systems that have a very
-        restrictive umask by default (e.g., "0077") and you want to pip install
+        restrictive umask by default (e.g., C(0077)) and you want to C(pip install)
         packages which are to be used by all users. Note that this requires you
-        to specify desired umask mode as an octal string, (e.g., "0022").
+        to specify desired umask mode as an octal string, (e.g., C(0022)).
     type: str
     version_added: "2.1"
   break_system_packages:
     description:
-      - Allow pip to modify an externally-managed Python installation as defined by PEP 668.
+      - Allow C(pip) to modify an externally-managed Python installation as defined by PEP 668.
       - This is typically required when installing packages outside a virtual environment on modern systems.
     type: bool
     default: false
@@ -134,7 +134,7 @@ notes:
      the virtualenv needs to be created.
    - Although it executes using the Ansible Python interpreter, the pip module shells out to
      run the actual pip command, so it can use any pip version you specify with O(executable).
-     By default, it uses the pip version for the Ansible Python interpreter. For example, pip3 on python 3, and pip2 or pip on python 2.
+     By default, it uses the pip version for the Ansible Python interpreter.
    - The interpreter used by Ansible
      (see R(ansible_python_interpreter, ansible_python_interpreter))
      requires the setuptools package, regardless of the version of pip set with
@@ -145,9 +145,9 @@ requirements:
 - setuptools or packaging
 author:
 - Matt Wright (@mattupstate)
-'''
+"""
 
-EXAMPLES = '''
+EXAMPLES = """
 - name: Install bottle python package
   ansible.builtin.pip:
     name: bottle
@@ -197,11 +197,11 @@ EXAMPLES = '''
     virtualenv: /my_app/venv
     virtualenv_site_packages: yes
 
-- name: Install bottle into the specified (virtualenv), using Python 2.7
+- name: Install bottle into the specified (virtualenv), using Python 3.13
   ansible.builtin.pip:
     name: bottle
     virtualenv: /my_app/venv
-    virtualenv_command: virtualenv-2.7
+    virtualenv_command: virtualenv-3.13
 
 - name: Install bottle within a user home directory
   ansible.builtin.pip:
@@ -227,10 +227,10 @@ EXAMPLES = '''
     requirements: /my_app/requirements.txt
     extra_args: "--no-index --find-links=file:///my_downloaded_packages_dir"
 
-- name: Install bottle for Python 3.3 specifically, using the 'pip3.3' executable
+- name: Install bottle for Python 3.13 specifically, using the 'pip3.13' executable
   ansible.builtin.pip:
     name: bottle
-    executable: pip3.3
+    executable: pip3.13
 
 - name: Install bottle, forcing reinstallation if it's already installed
   ansible.builtin.pip:
@@ -262,9 +262,9 @@ EXAMPLES = '''
   vars:
     venv_dir: /tmp/pick-a-better-venv-path
     venv_python: "{{ venv_dir }}/bin/python"
-'''
+"""
 
-RETURN = '''
+RETURN = """
 cmd:
   description: pip command used by the module
   returned: success
@@ -290,7 +290,7 @@ virtualenv:
   returned: success, if a virtualenv path was provided
   type: str
   sample: "/tmp/virtualenv"
-'''
+"""
 
 import argparse
 import os
@@ -299,7 +299,6 @@ import sys
 import tempfile
 import operator
 import shlex
-import traceback
 
 from ansible.module_utils.compat.version import LooseVersion
 
@@ -309,10 +308,10 @@ HAS_SETUPTOOLS = False
 try:
     from packaging.requirements import Requirement as parse_requirement
     HAS_PACKAGING = True
-except Exception:
+except Exception as ex:
     # This is catching a generic Exception, due to packaging on EL7 raising a TypeError on import
     HAS_PACKAGING = False
-    PACKAGING_IMP_ERR = traceback.format_exc()
+    PACKAGING_IMP_ERR = ex
     try:
         from pkg_resources import Requirement
         parse_requirement = Requirement.parse  # type: ignore[misc,assignment]
@@ -417,7 +416,7 @@ def _get_cmd_options(module, cmd):
 
 
 def _get_packages(module, pip, chdir):
-    '''Return results of pip command to get packages.'''
+    """Return results of pip command to get packages."""
     # Try 'pip list' command first.
     command = pip + ['list', '--format=freeze']
     locale = get_best_parsable_locale(module)
@@ -435,7 +434,7 @@ def _get_packages(module, pip, chdir):
 
 
 def _is_present(module, req, installed_pkgs, pkg_command):
-    '''Return whether or not package is installed.'''
+    """Return whether or not package is installed."""
     for pkg in installed_pkgs:
         if '==' in pkg:
             pkg_name, pkg_version = pkg.split('==')
@@ -461,9 +460,7 @@ def _get_pip(module, env=None, executable=None):
             candidate_pip_basenames = (executable,)
     elif executable is None and env is None and _have_pip_module():
         # If no executable or virtualenv were specified, use the pip module for the current Python interpreter if available.
-        # Use of `__main__` is required to support Python 2.6 since support for executing packages with `runpy` was added in Python 2.7.
-        # Without it Python 2.6 gives the following error: pip is a package and cannot be directly executed
-        pip = [sys.executable, '-m', 'pip.__main__']
+        pip = [sys.executable, '-m', 'pip']
 
     if pip is None:
         if env is None:
@@ -611,7 +608,7 @@ def setup_virtualenv(module, env, chdir, out, err):
     err += err_venv
     if rc != 0:
         _fail(module, cmd, out, err)
-    return out, err
+    return out, err, cmd
 
 
 class Package:
@@ -744,11 +741,12 @@ def main():
 
         err = ''
         out = ''
+        venv_cmd = ''
 
         if env:
             if not os.path.exists(os.path.join(env, 'bin', 'activate')):
                 venv_created = True
-                out, err = setup_virtualenv(module, env, chdir, out, err)
+                out, err, venv_cmd = setup_virtualenv(module, env, chdir, out, err)
             py_bin = os.path.join(env, 'bin', 'python')
         else:
             py_bin = module.params['executable'] or sys.executable
@@ -814,11 +812,14 @@ def main():
             cmd.extend(to_native(p) for p in packages)
         elif requirements:
             cmd.extend(['-r', requirements])
+        elif venv_created and not name and not requirements:
+            # ONLY creating an empty venv
+            module.exit_json(changed=venv_created, cmd=venv_cmd, name=name, version=version,
+                             state=state, requirements=requirements, virtualenv=env,
+                             stdout=out, stderr=err)
         else:
-            module.exit_json(
-                changed=False,
-                warnings=["No valid name or requirements file found."],
-            )
+            module.warn("No valid name or requirements file found.")
+            module.exit_json(changed=False)
 
         if module.check_mode:
             if extra_args or requirements or state == 'latest' or not name:
