@@ -1,4 +1,5 @@
 """Loading, parsing and storing of completion configurations."""
+
 from __future__ import annotations
 
 import abc
@@ -29,6 +30,7 @@ from .become import (
 
 class CGroupVersion(enum.Enum):
     """The control group version(s) required by a container."""
+
     NONE = 'none'
     V1_ONLY = 'v1-only'
     V2_ONLY = 'v2-only'
@@ -40,6 +42,7 @@ class CGroupVersion(enum.Enum):
 
 class AuditMode(enum.Enum):
     """The audit requirements of a container."""
+
     NONE = 'none'
     REQUIRED = 'required'
 
@@ -50,6 +53,7 @@ class AuditMode(enum.Enum):
 @dataclasses.dataclass(frozen=True)
 class CompletionConfig(metaclass=abc.ABCMeta):
     """Base class for completion configuration."""
+
     name: str
 
     @property
@@ -61,6 +65,7 @@ class CompletionConfig(metaclass=abc.ABCMeta):
 @dataclasses.dataclass(frozen=True)
 class PosixCompletionConfig(CompletionConfig, metaclass=abc.ABCMeta):
     """Base class for completion configuration of POSIX environments."""
+
     @property
     @abc.abstractmethod
     def supported_pythons(self) -> list[str]:
@@ -85,6 +90,7 @@ class PosixCompletionConfig(CompletionConfig, metaclass=abc.ABCMeta):
 @dataclasses.dataclass(frozen=True)
 class PythonCompletionConfig(PosixCompletionConfig, metaclass=abc.ABCMeta):
     """Base class for completion configuration of Python environments."""
+
     python: str = ''
     python_dir: str = '/usr/bin'
 
@@ -103,6 +109,7 @@ class PythonCompletionConfig(PosixCompletionConfig, metaclass=abc.ABCMeta):
 @dataclasses.dataclass(frozen=True)
 class RemoteCompletionConfig(CompletionConfig):
     """Base class for completion configuration of remote environments provisioned through Ansible Core CI."""
+
     provider: t.Optional[str] = None
     arch: t.Optional[str] = None
 
@@ -132,6 +139,7 @@ class RemoteCompletionConfig(CompletionConfig):
 @dataclasses.dataclass(frozen=True)
 class InventoryCompletionConfig(CompletionConfig):
     """Configuration for inventory files."""
+
     def __init__(self) -> None:
         super().__init__(name='inventory')
 
@@ -144,6 +152,7 @@ class InventoryCompletionConfig(CompletionConfig):
 @dataclasses.dataclass(frozen=True)
 class PosixSshCompletionConfig(PythonCompletionConfig):
     """Configuration for a POSIX host reachable over SSH."""
+
     def __init__(self, user: str, host: str) -> None:
         super().__init__(
             name=f'{user}@{host}',
@@ -159,6 +168,7 @@ class PosixSshCompletionConfig(PythonCompletionConfig):
 @dataclasses.dataclass(frozen=True)
 class DockerCompletionConfig(PythonCompletionConfig):
     """Configuration for Docker containers."""
+
     image: str = ''
     seccomp: str = 'default'
     cgroup: str = CGroupVersion.V1_V2.value
@@ -201,6 +211,7 @@ class DockerCompletionConfig(PythonCompletionConfig):
 @dataclasses.dataclass(frozen=True)
 class NetworkRemoteCompletionConfig(RemoteCompletionConfig):
     """Configuration for remote network platforms."""
+
     collection: str = ''
     connection: str = ''
     placeholder: bool = False
@@ -213,6 +224,7 @@ class NetworkRemoteCompletionConfig(RemoteCompletionConfig):
 @dataclasses.dataclass(frozen=True)
 class PosixRemoteCompletionConfig(RemoteCompletionConfig, PythonCompletionConfig):
     """Configuration for remote POSIX platforms."""
+
     become: t.Optional[str] = None
     placeholder: bool = False
 
@@ -235,11 +247,10 @@ class PosixRemoteCompletionConfig(RemoteCompletionConfig, PythonCompletionConfig
 class WindowsRemoteCompletionConfig(RemoteCompletionConfig):
     """Configuration for remote Windows platforms."""
 
-
-TCompletionConfig = t.TypeVar('TCompletionConfig', bound=CompletionConfig)
+    connection: str = ''
 
 
-def load_completion(name: str, completion_type: t.Type[TCompletionConfig]) -> dict[str, TCompletionConfig]:
+def load_completion[TCompletionConfig: CompletionConfig](name: str, completion_type: t.Type[TCompletionConfig]) -> dict[str, TCompletionConfig]:
     """Load the named completion entries, returning them in dictionary form using the specified completion type."""
     lines = read_lines_without_comments(os.path.join(ANSIBLE_TEST_DATA_ROOT, 'completion', '%s.txt' % name), remove_blank_lines=True)
 
@@ -269,7 +280,7 @@ def parse_completion_entry(value: str) -> tuple[str, dict[str, str]]:
     return name, data
 
 
-def filter_completion(
+def filter_completion[TCompletionConfig: CompletionConfig](
     completion: dict[str, TCompletionConfig],
     controller_only: bool = False,
     include_defaults: bool = False,
